<?php

/*
 * This controller polls Prometheus and node_exporter data using functions from
 * the helper 'blueonyx_helper.php'. In order for this to work, the services
 * 'prometheus.service' and 'node_exporter.service' must be running.
*/

namespace Gui\Controllers;
use App\Controllers\BaseController;
include_once("I18n.php");
require_once '/usr/sausalito/ui/chorizo/ci4/app/Helpers/blueonyx_helper.php';
use I18n;

class Metrics extends BaseController {

    public function __construct() {

    }

    public function index() {

        $CI =& get_instance();
        $BX_SESSION = $CI->getBX_SESSION();

        // Only logged in users with 'serverShowActiveMonitor' get access to metrics:
        if (!$CI->getAllowed('serverShowActiveMonitor')) {
            // Nice people say goodbye, or CCEd waits forever:
            $CI->cceClient->bye();
            $CI->serverScriptHelper->destructor();
            Log403Error("/gui/Forbidden403");
        }

        $factory = $CI->serverScriptHelper->getHtmlComponentFactory("base-alpine", "/gui/metrics");
        $BxPage = $factory->getPage();
        $BxPage->setGETPOST(array('FORM_GET' => $this->request->getGet(), 'FORM_POST' => $this->request->getPost(), 'AGENT' => $this->request->getUserAgent()));
        $get_form_data = $BxPage->getGETPOST('GET');

        // Return ALL metrics:
        $query = '';
        $squery = '';

        // Or do we have a query string and return selected metrics only?
        if (isset($get_form_data['q'])) {
            $query = $get_form_data['q'];
        }

        if ($query === 'shortload') {
            $squery = $query;
            $query = 'node_load';
        }

        if ($query === 'shortmem') {
            $squery = $query;
            $query = 'node_memory';
        }

        if ($query === 'shortnet') {
            $squery = $query;
            $query = 'node_network';
        }

        if ($query === 'aggnet') {
            $squery = $query;
            $query = 'node_network';
        }

        if ($query === 'shortdisk') {
            $squery = $query;
            $query = 'node_disk_io_now';
        }

        // Logic to retrieve and format metrics data:
        $metricsData = getMetricsData($query);

        if ($squery === 'shortload') {

            $desiredMetrics = [
                'node_load1',
                'node_load5',
                'node_load15',
            ];

            $resultArray = [];

            foreach ($metricsData as $item) {
                if (in_array($item['name'], $desiredMetrics)) {
                    $resultArray[] = $item;
                }
            }

            // Sorting done the stupid way:
            $metricsData = [];
            $metricsData[] = $resultArray['0'];
            $metricsData[] = $resultArray['2'];
            $metricsData[] = $resultArray['1'];
        }

        if ($squery === 'shortmem') {

            $desiredMetrics = [
                'node_memory_MemTotal_bytes',
                'node_memory_MemAvailable_bytes',
                'node_memory_MemFree_bytes',
                'node_memory_SwapTotal_bytes',
                'node_memory_SwapFree_bytes',
            ];

            $resultArray = [];

            foreach ($metricsData as $item) {
                if (in_array($item['name'], $desiredMetrics)) {
                    $resultArray[] = $item;
                }
            }

            $metricsData = $resultArray;
        }

        if ($squery === 'shortnet') {
            $desiredMetrics = [
                'node_network_transmit_bytes_total',
                'node_network_receive_bytes_total',
            ];

            $aggregatedMetrics = [];

            foreach ($metricsData as $metric) {
                // Check if the metric is in the desired list and not from 'lo' device
                foreach ($desiredMetrics as $desiredMetric) {
                    $pattern = "/$desiredMetric\{device=\"([^\"]+)\"\}/";
                    if (preg_match($pattern, $metric['name'], $matches) && !preg_match('/device="lo"/', $metric['name'])) {
                        // Replace {device="..."} with just the device name and remove 'node_network_'
                        $metricNameWithoutDevice = str_replace(['{device="', '"}', 'node_network_'], ['_', '', ''], $metric['name']);
                        // Combine the metric name without device and the device name
                        $aggregatedMetrics[] = [
                            'name' => $metricNameWithoutDevice,
                            'value' => $metric['value'],
                        ];
                        // Break out of the inner loop since we found a match
                        break;
                    }
                }
            }
            $metricsData = [];
            $metricsData[] = $aggregatedMetrics[1];
            $metricsData[] = $aggregatedMetrics[0];
        }

        if ($squery === 'aggnet') {
            $desiredMetrics = [
                'node_network_transmit_bytes_total',
                'node_network_receive_bytes_total',
            ];

            $have_historic_data = FALSE;

            if (session()->get('aggnet')) {
                $session_aggnet = session()->get('aggnet');
                $have_historic_data = TRUE;

                if (isset($session_aggnet['timestamp'])) {
                    $currentTimestamp = time();
                    $timeDiff = $currentTimestamp - (int)$session_aggnet['timestamp'];
                    if ($timeDiff >= 15) {
                        $have_historic_data = FALSE;
                        $session_aggnet = [];
                    }
                }
                else {
                    $have_historic_data = FALSE;
                    $session_aggnet = [];
                }
            }
            else {
                $have_historic_data = FALSE;
                $session_aggnet = [];
            }

            $aggregatedMetrics = [];
            $newest_data = [];
            $data = [];
            $differential_output = [];

            foreach ($metricsData as $metric) {
                // Check if the metric is in the desired list and not from 'lo' device
                foreach ($desiredMetrics as $desiredMetric) {
                    $pattern = "/$desiredMetric\{device=\"([^\"]+)\"\}/";
                    if (preg_match($pattern, $metric['name'], $matches) && !preg_match('/device="lo"/', $metric['name'])) {
                        // Replace {device="..."} with just the device name and remove 'node_network_'
                        $metricNameWithoutDevice = str_replace(['{device="', '"}', 'node_network_'], ['_', '', ''], $metric['name']);
                        $aggregatedMetrics[$metricNameWithoutDevice] = $metric['value'];

                        if ($have_historic_data === FALSE) {
                            $session_aggnet[$metricNameWithoutDevice] = $metric['value'];
                            $session_aggnet['timestamp'] = time();
                        }

                        // Break out of the inner loop since we found a match
                        break;
                    }
                }
            }

            $data['aggnet'] = $aggregatedMetrics;
            $data['aggnet']['timestamp'] = time();
            session()->set($data);

            // Calculate diffs:
            foreach ($aggregatedMetrics as $key => $value) {
                if ((isset($aggregatedMetrics[$key])) && (isset($session_aggnet[$key]))) {
                    $diff_data = $aggregatedMetrics[$key] - $session_aggnet[$key];
                    $differential_output[] = array('name' => $key, 'labels' => [], 'value' => $diff_data);
                }
            }

            // Set Headers to no caching at all:
            $this->response->noCache();

            // Return metrics as JSON
            return $this->response->setJSON($differential_output);
        }

        if ($squery === 'shortdisk') {
            $desiredMetrics = [
                'node_disk_io_now',
            ];

            $aggregatedMetrics = [];

            foreach ($metricsData as $metric) {
                // Check if the metric is in the desired list and not from 'lo' device
                foreach ($desiredMetrics as $desiredMetric) {
                    $pattern = "/$desiredMetric\{device=\"([^\"]+)\"\}/";
                    if (preg_match($pattern, $metric['name'], $matches)) {
                        // Replace {device="..."} with just the device name and remove 'node_network_'
                        $metricNameWithoutDevice = str_replace(['{device="', '"}', 'node_'], ['_', '', ''], $metric['name']);
                        // Combine the metric name without device and the device name
                        $aggregatedMetrics[] = [
                            'name' => $metricNameWithoutDevice,
                            'value' => $metric['value'],
                        ];
                        // Break out of the inner loop since we found a match
                        break;
                    }
                }
            }
            $metricsData = $aggregatedMetrics;
        }

        // Set Headers to no caching at all:
        $this->response->noCache();

        // Return metrics as JSON
        return $this->response->setJSON($metricsData);
    }
}

/*
Copyright (c) 2008-2023 Michael Stauber, SOLARSPEED.NET
Copyright (c) 2008-2023 Team BlueOnyx, BLUEONYX.IT
All Rights Reserved.

1. Redistributions of source code must retain the above copyright 
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright 
   notice, this list of conditions and the following disclaimer in 
   the documentation and/or other materials provided with the 
   distribution.

3. Neither the name of the copyright holder nor the names of its 
   contributors may be used to endorse or promote products derived 
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.

You acknowledge that this software is not designed or intended for 
use in the design, construction, operation or maintenance of any 
nuclear facility.

*/
?>